unit FileSplitter;

{                 TFileSplitter Component
                   1999 by Christoph Mssel
     Component to split a big file to several smaller files
                   and to reassemble them.

              Mailto Christoph.Muessel@t-online.de
                http://private.freepage.de/muessel

                 Properties, Methods and Events:

***Properties***
AssembledFileName: Indicates the file name of the file that is not splitted.
SplittedFileName: Indicates the file name of the splitted files (no extension!)
MaxFileSize: The maximum file size of a splitted file in bytes. Normally, the
             size of a floppy disk.

***Methods***
SplitFile: Splits the file whose name is given in AssembledFileName to multiple files.
AssembleFile: Assembles several files to one file whose name is the AssembledFileName property.

***Events***
OnChangeDisk(Sender:TObject, NumDisk:word): This Event is called when the disk is full or
                             when the file size is MaxFileSize. The NumDisk parameter in-
                             dicates the number of the next disk.
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TChangeDiskEvent = procedure(Sender:TObject; DiskNum:word) of object;

  TFileSplitter = class(TComponent)
  private
    { Private-Deklarationen }
    FAssembledFileName:string;
    FSplittedFileName:string;
    FMaxFileSize:LongInt;
    FOnChangeDisk:TChangeDiskEvent;
  protected
    { Protected-Deklarationen }
  public
    { Public-Deklarationen }
    constructor Create(AOwner:TComponent);override;
    procedure SplitFile;
    procedure AssembleFile;
  published
    { Published-Deklarationen }
    property AssembledFileName:string read FAssembledFileName write FAssembledFileName;
    property SplittedFileName:string read FSplittedFileName write FSplittedFileName;
    property MaxFileSize:LongInt read FMaxFileSize write FMaxFileSize default 1457664;
    property OnChangeDisk:TChangeDiskEvent read FOnChangeDisk write FOnChangeDisk;
  end;

procedure Register;

implementation
{$R FileSplitter.dcr}
var
  FromF, ToF: file;
  NumRead, NumWritten: Integer;
  Buf: array[1..2048] of Char;
  ND, drivenum:byte;
  diskNum:word;
  Space, Ges, Max:LongInt;
  drive:string;

procedure Register;
begin
  RegisterComponents('FileTools', [TFileSplitter]);
end;

procedure TFileSplitter.SplitFile;
begin
Drive:=LowerCase(ExtractFileDrive(FSplittedFileName));
DriveNum:=Ord(Drive[1])-96;
DiskNum:=0;
AssignFile(FromF, FAssembledFileName);
Reset(FromF, 1);
Ges:=0;
repeat;
If (DiskNum<>0) then
If Assigned(FOnChangeDisk) then FOnChangeDisk(Self,DiskNum+1)
else
If (DiskFree(DriveNum)<=2) then
showMessage('Please insert disk No. '+inttostr(DiskNum+1)+'!');
inc(Disknum);
AssignFile(ToF, FSplittedFileName+'.'+inttostr(DiskNum));
Rewrite(ToF, 1);
ND:=DiskNum;
Space:=DiskFree(DriveNum)-1;
If (Space>FMaxFileSize-1) then Space:=FMaxFileSize-1;
If (Space>=FileSize(FromF)-Ges) then ND:=255;
BlockWrite(ToF,ND,1);
repeat
If (Space>SizeOf(Buf)) then Max:=SizeOf(Buf) else Max:=Space;
BlockRead(FromF, Buf, Max, NumRead);
BlockWrite(ToF, Buf, NumRead, NumWritten);
Space:=Space-NumWritten;
inc(Ges,NumWritten);
until (Space=0) or (Ges=FileSize(FromF));
CloseFile(ToF);
Until (Ges=FileSize(FromF));
CloseFile(FromF);
end;

procedure TFileSplitter.AssembleFile;
Label 1;
begin
DiskNum:=0;
AssignFile(ToF, AssembledFileName);
Rewrite(ToF, 1);
repeat;
Ges:=0;
inc(Disknum);
1:
If not FileExists(SplittedFileName+'.'+inttostr(DiskNum)) then
begin;
If Assigned(FOnChangeDisk) then FOnChangeDisk(Self,DiskNum)
else
showMessage('Please insert disk No. '+inttostr(DiskNum)+'!');
Goto 1;
end;
AssignFile(FromF, SplittedFileName+'.'+inttostr(DiskNum));
Reset(FromF, 1);
Space:=FileSize(FromF)-1;
BlockRead(FromF,ND,1);
repeat
If (Space>SizeOf(Buf)) then Max:=SizeOf(Buf) else Max:=Space;
BlockRead(FromF, Buf, Max, NumRead);
BlockWrite(ToF, Buf, NumRead, NumWritten);
inc(Ges,NumWritten);
until (Ges=Space);
CloseFile(FromF);
Until (ND=255);
CloseFile(ToF);
end;

constructor TFileSplitter.Create(AOwner:TComponent);
begin
inherited Create(AOwner);
FMaxFileSize:=1457664;
FSplittedFileName:='A:\splitted';
end;

end.
